@@ -1,5 +1,5 @@
-from sympy.core.backend import (S, sympify, expand, sqrt, Add, zeros, acos,
-                                ImmutableMatrix as Matrix, _simplify_matrix)
+from sympy import (S, sympify, expand, sqrt, Add, zeros, acos,
+                   ImmutableMatrix as Matrix, simplify)
 from sympy.simplify.trigsimp import trigsimp
 from sympy.printing.defaults import Printable
 from sympy.utilities.misc import filldedent
@@ -29,7 +29,7 @@ class Vector(Printable, EvalfMixin):
     is_number = False
 
     def __init__(self, inlist):
-        """This is the constructor for the Vector class.  You should not be
+        """This is the constructor for the Vector class. You should not be
         calling this, it should only be used by other functions. You should be
         treating Vectors like you would with if you were doing the math by
         hand, and getting the first 3 from the standard basis vectors from a
@@ -72,7 +72,7 @@ class Vector(Printable, EvalfMixin):
         other = _check_vector(other)
         return Vector(self.args + other.args)
 
-    def __and__(self, other):
+    def dot(self, other):
         """Dot product of two vectors.
 
         Returns a scalar, the dot product of the two Vectors
@@ -100,16 +100,18 @@ class Vector(Printable, EvalfMixin):
 
         """
 
-        from sympy.physics.vector.dyadic import Dyadic
+        from sympy.physics.vector.dyadic import Dyadic, _check_dyadic
         if isinstance(other, Dyadic):
-            return NotImplemented
+            other = _check_dyadic(other)
+            ol = Vector(0)
+            for v in other.args:
+                ol += v[0] * v[2] * (v[1].dot(self))
+            return ol
         other = _check_vector(other)
         out = S.Zero
-        for i, v1 in enumerate(self.args):
-            for j, v2 in enumerate(other.args):
-                out += ((v2[0].T)
-                        * (v2[1].dcm(v1[1]))
-                        * (v1[0]))[0]
+        for v1 in self.args:
+            for v2 in other.args:
+                out += ((v2[0].T) * (v2[1].dcm(v1[1])) * (v1[0]))[0]
         if Vector.simp:
             return trigsimp(out, recursive=True)
         else:
@@ -144,7 +146,7 @@ class Vector(Printable, EvalfMixin):
 
         frame = self.args[0][1]
         for v in frame:
-            if expand((self - other) & v) != 0:
+            if expand((self - other).dot(v)) != 0:
                 return False
         return True
 
@@ -170,7 +172,7 @@ class Vector(Printable, EvalfMixin):
 
         """
 
-        newlist = [v for v in self.args]
+        newlist = list(self.args)
         other = sympify(other)
         for i, v in enumerate(newlist):
             newlist[i] = (other * newlist[i][0], newlist[i][1])
@@ -179,7 +181,7 @@ class Vector(Printable, EvalfMixin):
     def __neg__(self):
         return self * -1
 
-    def __or__(self, other):
+    def outer(self, other):
         """Outer product between two Vectors.
 
         A rank increasing operation, which returns a Dyadic from two Vectors
@@ -203,8 +205,8 @@ class Vector(Printable, EvalfMixin):
         from sympy.physics.vector.dyadic import Dyadic
         other = _check_vector(other)
         ol = Dyadic(0)
-        for i, v in enumerate(self.args):
-            for i2, v2 in enumerate(other.args):
+        for v in self.args:
+            for v2 in other.args:
                 # it looks this way because if we are in the same frame and
                 # use the enumerate function on the same frame in a nested
                 # fashion, then bad things happen
@@ -256,89 +258,28 @@ class Vector(Printable, EvalfMixin):
     def _pretty(self, printer):
         """Pretty Printing method. """
         from sympy.printing.pretty.stringpict import prettyForm
-        e = self
 
-        class Fake:
+        terms = []
 
-            def render(self, *args, **kwargs):
-                ar = e.args  # just to shorten things
-                if len(ar) == 0:
-                    return str(0)
-                pforms = []  # output list, to be concatenated to a string
-                for i, v in enumerate(ar):
-                    for j in 0, 1, 2:
-                        # if the coef of the basis vector is 1, we skip the 1
-                        if ar[i][0][j] == 1:
-                            pform = printer._print(ar[i][1].pretty_vecs[j])
-                        # if the coef of the basis vector is -1, we skip the 1
-                        elif ar[i][0][j] == -1:
-                            pform = printer._print(ar[i][1].pretty_vecs[j])
-                            pform = prettyForm(*pform.left(" - "))
-                            bin = prettyForm.NEG
-                            pform = prettyForm(binding=bin, *pform)
-                        elif ar[i][0][j] != 0:
-                            # If the basis vector coeff is not 1 or -1,
-                            # we might wrap it in parentheses, for readability.
-                            pform = printer._print(ar[i][0][j])
+        def juxtapose(a, b):
+            pa = printer._print(a)
+            pb = printer._print(b)
+            if a.is_Add:
+                pa = prettyForm(*pa.parens())
+            return printer._print_seq([pa, pb], delimiter=' ')
 
-                            if isinstance(ar[i][0][j], Add):
-                                tmp = pform.parens()
-                                pform = prettyForm(tmp[0], tmp[1])
+        for M, N in self.args:
+            for i in range(3):
+                if M[i] == 0:
+                    continue
+                elif M[i] == 1:
+                    terms.append(prettyForm(N.pretty_vecs[i]))
+                elif M[i] == -1:
+                    terms.append(prettyForm("-1") * prettyForm(N.pretty_vecs[i]))
+                else:
+                    terms.append(juxtapose(M[i], N.pretty_vecs[i]))
 
-                            pform = prettyForm(*pform.right(
-                                " ", ar[i][1].pretty_vecs[j]))
-                        else:
-                            continue
-                        pforms.append(pform)
-
-                pform = prettyForm.__add__(*pforms)
-                kwargs["wrap_line"] = kwargs.get("wrap_line")
-                kwargs["num_columns"] = kwargs.get("num_columns")
-                out_str = pform.render(*args, **kwargs)
-                mlines = [line.rstrip() for line in out_str.split("\n")]
-                return "\n".join(mlines)
-
-        return Fake()
-
-    def __ror__(self, other):
-        """Outer product between two Vectors.
-
-        A rank increasing operation, which returns a Dyadic from two Vectors
-
-        Parameters
-        ==========
-
-        other : Vector
-            The Vector to take the outer product with
-
-        Examples
-        ========
-
-        >>> from sympy.physics.vector import ReferenceFrame, outer
-        >>> N = ReferenceFrame('N')
-        >>> outer(N.x, N.x)
-        (N.x|N.x)
-
-        """
-
-        from sympy.physics.vector.dyadic import Dyadic
-        other = _check_vector(other)
-        ol = Dyadic(0)
-        for i, v in enumerate(other.args):
-            for i2, v2 in enumerate(self.args):
-                # it looks this way because if we are in the same frame and
-                # use the enumerate function on the same frame in a nested
-                # fashion, then bad things happen
-                ol += Dyadic([(v[0][0] * v2[0][0], v[1].x, v2[1].x)])
-                ol += Dyadic([(v[0][0] * v2[0][1], v[1].x, v2[1].y)])
-                ol += Dyadic([(v[0][0] * v2[0][2], v[1].x, v2[1].z)])
-                ol += Dyadic([(v[0][1] * v2[0][0], v[1].y, v2[1].x)])
-                ol += Dyadic([(v[0][1] * v2[0][1], v[1].y, v2[1].y)])
-                ol += Dyadic([(v[0][1] * v2[0][2], v[1].y, v2[1].z)])
-                ol += Dyadic([(v[0][2] * v2[0][0], v[1].z, v2[1].x)])
-                ol += Dyadic([(v[0][2] * v2[0][1], v[1].z, v2[1].y)])
-                ol += Dyadic([(v[0][2] * v2[0][2], v[1].z, v2[1].z)])
-        return ol
+        return prettyForm.__add__(*terms)
 
     def __rsub__(self, other):
         return (-1 * self) + other
@@ -387,7 +328,7 @@ class Vector(Printable, EvalfMixin):
         """The subtraction operator. """
         return self.__add__(other * -1)
 
-    def __xor__(self, other):
+    def cross(self, other):
         """The cross product operator for two Vectors.
 
         Returns a Vector, expressed in the same ReferenceFrames as self.
@@ -416,9 +357,13 @@ class Vector(Printable, EvalfMixin):
 
         """
 
-        from sympy.physics.vector.dyadic import Dyadic
+        from sympy.physics.vector.dyadic import Dyadic, _check_dyadic
         if isinstance(other, Dyadic):
-            return NotImplemented
+            other = _check_dyadic(other)
+            ol = Dyadic(0)
+            for i, v in enumerate(other.args):
+                ol += v[0] * ((self.cross(v[1])).outer(v[2]))
+            return ol
         other = _check_vector(other)
         if other.args == []:
             return Vector(0)
@@ -443,14 +388,13 @@ class Vector(Printable, EvalfMixin):
             tempy = v[1].y
             tempz = v[1].z
             tempm = ([[tempx, tempy, tempz],
-                      [self & tempx, self & tempy, self & tempz],
-                      [Vector([ar[i]]) & tempx, Vector([ar[i]]) & tempy,
-                       Vector([ar[i]]) & tempz]])
+                      [self.dot(tempx), self.dot(tempy), self.dot(tempz)],
+                      [Vector([ar[i]]).dot(tempx), Vector([ar[i]]).dot(tempy),
+                       Vector([ar[i]]).dot(tempz)]])
             outlist += _det(tempm).args
         return Vector(outlist)
 
     __radd__ = __add__
-    __rand__ = __and__
     __rmul__ = __mul__
 
     def separate(self):
@@ -478,17 +422,18 @@ class Vector(Printable, EvalfMixin):
             components[x[1]] = Vector([x])
         return components
 
-    def dot(self, other):
-        return self & other
-    dot.__doc__ = __and__.__doc__
+    def __and__(self, other):
+        return self.dot(other)
+    __and__.__doc__ = dot.__doc__
+    __rand__ = __and__
 
-    def cross(self, other):
-        return self ^ other
-    cross.__doc__ = __xor__.__doc__
+    def __xor__(self, other):
+        return self.cross(other)
+    __xor__.__doc__ = cross.__doc__
 
-    def outer(self, other):
-        return self | other
-    outer.__doc__ = __or__.__doc__
+    def __or__(self, other):
+        return self.outer(other)
+    __or__.__doc__ = outer.__doc__
 
     def diff(self, var, frame, var_in_dcm=True):
         """Returns the partial derivative of the vector with respect to a
@@ -513,17 +458,15 @@ class Vector(Printable, EvalfMixin):
 
         >>> from sympy import Symbol
         >>> from sympy.physics.vector import dynamicsymbols, ReferenceFrame
-        >>> from sympy.physics.vector import Vector
         >>> from sympy.physics.vector import init_vprinting
         >>> init_vprinting(pretty_print=False)
-        >>> Vector.simp = True
         >>> t = Symbol('t')
         >>> q1 = dynamicsymbols('q1')
         >>> N = ReferenceFrame('N')
         >>> A = N.orientnew('A', 'Axis', [q1, N.y])
         >>> A.x.diff(t, N)
         - sin(q1)*q1'*N.x - cos(q1)*q1'*N.z
-        >>> A.x.diff(t, N).express(A)
+        >>> A.x.diff(t, N).express(A).simplify()
         - q1'*A.z
         >>> B = ReferenceFrame('B')
         >>> u1, u2 = dynamicsymbols('u1, u2')
@@ -657,7 +600,7 @@ class Vector(Printable, EvalfMixin):
         """Returns a simplified Vector."""
         d = {}
         for v in self.args:
-            d[v[1]] = _simplify_matrix(v[0])
+            d[v[1]] = simplify(v[0])
         return Vector(d)
 
     def subs(self, *args, **kwargs):
@@ -693,7 +636,7 @@ class Vector(Printable, EvalfMixin):
         instead of ``(-A.x).magnitude()``.
 
         """
-        return sqrt(self & self)
+        return sqrt(self.dot(self))
 
     def normalize(self):
         """Returns a Vector of magnitude 1, codirectional with self."""
