@@ -1080,3 +1080,8 @@ def test_reduce_inverses_nc_pow():
     x, y = symbols("x y", positive=True)
     assert expand((x*y)**Z) == x**Z * y**Z
     assert simplify(x**Z * y**Z) == expand((x*y)**Z)
+
+def test_nc_recursion_coeff():
+    X = symbols("X", commutative = False)
+    assert (2 * cos(pi/3) * X).simplify() == X
+    assert (2.0 * cos(pi/3) * X).simplify() == X
